/**********************************************************************************
 *    EasyPCA performs a PCA analysis of a binary input file                      *
 *    Copyright (C) 2008  Javier Cruz Mota                                        *
 *    E-Mail: javier.cruz (AT) epfl.ch                                            *
 *    Postal Address: EPFL /  ENAC /  INTER /  TRANSP-OR                          *
 *                    GC B3 425 (Bâtiment GC), Station 18                         *
 *                    CH-1015 Lausanne (Switzerland)                              *
 *                                                                                *
 *    This file is part of EasyPCA.                                               * 
 *                                                                                *
 *    EasyPCA is free software; you can redistribute it and/or modify             *
 *    it under the terms of the GNU General Public License as published by        *
 *    the Free Software Foundation; either version 2 of the License, or           *
 *    (at your option) any later version.                                         *
 *                                                                                *
 *    EasyPCA is distributed in the hope that it will be useful,                  *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *    GNU General Public License for more details.                                *
 *                                                                                *
 *    You should have received a copy of the GNU General Public License           *
 *    along with EasyPCA; if not, write to the Free Software                      *
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  *
 **********************************************************************************/

#ifndef PCA_HPP_
#define PCA_HPP_

#include <iostream>
#include <cmath>
#include <fstream>
#include <string>
#include <cstdlib>

#include "utils.hpp"

const std::string OpenERROR               = "Error while opening file";
const std::string TooManyIterationERROR   = "Too many iterations";
const std::string InvalidEnergyValueERROR = "Invalid energy value to be selected";

const int ERROR = -1;
const int NOERROR = 0;

const int DEBUG = 0;

class PCA
{
	public:
		PCA(){};
		~PCA(){};
		void PCA_LoadMatrix(void);
		void PCA_LoadMatrix(const char *PCA_File);
		void PCA_PerformPCA(void);
		void PCA_Transpose(void);
		void PCA_DeTranspose(void);
		void PCA_Print(void);
		void PCA_PrintResult(void);
		void PCA_Print(const char *PCA_str);
		void PCA_SelectSubset(double selEnergy, bool isTransp);
		void PCA_WriteResult(const char *OUT_File);
		void PCA_WriteC(const char *OUT_File);
		void PCA_WriteB(const char *OUT_File);
		void PCA_WriteX(const char *OUT_File);
		int PCA_GetN(void){return PCA_N;};
		int PCA_GetM(void){return PCA_M;};
		bool PCA_GetIsTransposed(void){return PCA_IsTransposed;};
	private:
		double **PCA_X;
		double *PCA_u;
		double **PCA_B;
		double **PCA_C;
		double *PCA_d;
		double *PCA_e;
		double *PCA_g;
		int PCA_N;
		int PCA_M;
		int PCA_indexEnergy;
		bool PCA_IsTransposed;
		void PCA_EmpiricalMean(void);
		void PCA_SubstractMean(void);
		void PCA_CovarianceMatrix(void);
		void PCA_HouseHolderTridiag(void);
		void PCA_QL(void);
		void PCA_QuickSortEV(void);
		void PCA_QuickSort(double numbers[], int index[], int left, int right);
		void PCA_CumulativeEnergy(void);
		void PCA_Nomalization(void);
};

#endif /*PCA_HPP_*/
